/* Module to read file more efficiently.
 *
 * `bufferfd` reduces the number of calls to `read` by maintaining a buffer.
 *
 * Usage:
 *
 * ```c
 * struct bufferfd file;
 * file.fd = fd;
 * file.len = 0;
 * file.pos = 0;
 *
 * // ...use the buffer...
 * ```
 *
 * Return value of the functions:
 * - On success, the number of bytes consumed (except bufferfd_seek returns 0)
 * - On error, a negative value
 *
 * Note: the user should not interfere with the members of `bufferfd` once
 * initialized.
 */
#if !defined(LKT_BUFFERFD_H__)
#define LKT_BUFFERFD_H__

#if defined(__cplusplus)
extern "C" {
#endif

#include <lektor/common.h>

struct bufferfd {
    int fd;
    uint8_t buffer[BUFFER_MAX];
    size_t len;
    size_t pos;
};

/* Read `n` bytes and copy them to `res`. */
ssize_t bufferfd_bytes(struct bufferfd *bf, size_t n, uint8_t *res);

/* Read a byte and copy it to `res`.
 * Equivalent to `bufferfd_bytes(bf, 1, res)`. */
ssize_t bufferfd_byte(struct bufferfd *bf, uint8_t *res);

/* Consume `n` bytes without returning them. */
ssize_t bufferfd_skip(struct bufferfd *bf, const size_t n);

/* Move the file cursor to the absolute position `pos` (0 is the beginning of
 * the file).
 * Internally calls `lseek(bf->fd, pos, SEEK_SET)`. */
ssize_t bufferfd_seek(struct bufferfd *bf, size_t pos);

#if defined(__cplusplus)
}
#endif
#endif /* LKT_BUFFERFD_H__ */
