#pragma once

#if defined(__cplusplus)
extern "C" {
#endif

#include <inttypes.h>
#include <locale.h>
#include <langinfo.h>
#include <memory.h>
#include <regex.h>
#include <stddef.h>
#include <unistd.h>
#include <stdint.h>
#include <stdlib.h>
#include <strings.h>
#include <string.h>
#include <memory.h>
#include <limits.h>
#include <stdbool.h>
#include <stdarg.h>
#include <errno.h>
#include <pthread.h>
#include <stdio.h>
#include <time.h>
#include <ctype.h>

#if defined(LKT_OS_TOASTER) && (LKT_OS_TOASTER == 1)
#include <linux/limits.h>
#include <dirent.h>
#include <fcntl.h>
#include <endian.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/time.h>
#endif

#if defined(LKT_OS_WIN) && (LKT_OS_WIN == 1)
#include "lektor/internal/win32/dirent.h"
#include "lektor/internal/win32/endian.h"
#include "lektor/internal/win32/termios.h"
#endif

#if !(__BYTE_ORDER == __LITTLE_ENDIAN || __BYTE_ORDER == __BIG_ENDIAN)
#error "Unsuported endiannes"
#endif

/* File types for `d_type'. */
#if !defined(DT_UNKNOWN) && !defined(DT_FIFO) && !defined(DT_CHR) && !defined(DT_DIR) && \
    !defined(DT_BLK) && !defined(DT_REG) && !defined(DT_LNK) && !defined(DT_SOCK) &&     \
    !defined(DT_WHT)
enum {
    DT_UNKNOWN = 0,
    DT_FIFO    = 1,
    DT_CHR     = 2,
    DT_DIR     = 4,
    DT_BLK     = 6,
    DT_REG     = 8,
    DT_LNK     = 10,
    DT_SOCK    = 12,
    DT_WHT     = 14,
};
#define DT_UNKNOWN DT_UNKNOWN
#define DT_FIFO    DT_FIFO
#define DT_CHR     DT_CHR
#define DT_DIR     DT_DIR
#define DT_BLK     DT_BLK
#define DT_REG     DT_REG
#define DT_LNK     DT_LNK
#define DT_SOCK    DT_SOCK
#define DT_WHT     DT_WHT
#endif

typedef void (*function_ptr)(void);

/* Attributes */
#define noreturn             __attribute__((noreturn))
#define UNUSED               __attribute__((unused))
#define UNUSED_FUNCTION      UNUSED
#define CONSTRUCTOR_FUNCTION __attribute__((constructor)) static void
#define DESTRUCTOR_FUNCTION  __attribute__((destructor)) static void
#define EXIT_FUNCTION        noreturn void
#define PRIVATE_FUNCTION     UNUSED_FUNCTION static inline
#define FALLTHROUGH          __attribute__((fallthrough))

/* Compilers */
#if defined(__clang__)
#define LKT_PUSH                     _Pragma("clang diagnostic push")
#define LKT_IGNORE_WARN_FMT_SECURITY _Pragma("clang diagnostic ignored \"-Wformat-security\"")
#define LKT_IGNORE_WARN_WRITE_STRING _Pragma("clang diagnostic ignored \"-Wwrite-strings\"")
#define LKT_IGNORE_WARN_FALLTHROUGH  _Pragma("clang diagnostic ignored \"-Wimplicit-fallthrough\"")
#define LKT_POP                      _Pragma("clang diagnostic pop")

#elif defined(__GNUC__) || defined(__GNUG__)
#define LKT_PUSH                     _Pragma("GCC diagnostic push")
#define LKT_IGNORE_WARN_FMT_SECURITY _Pragma("GCC diagnostic ignored \"-Wformat-security\"")
#define LKT_IGNORE_WARN_WRITE_STRING _Pragma("GCC diagnostic ignored \"-Wwrite-strings\"")
#define LKT_IGNORE_WARN_FALLTHROUGH  _Pragma("GCC diagnostic ignored \"-Wimplicit-fallthrough\"")
#define LKT_POP                      _Pragma("GCC diagnostic pop")

#elif defined(_MSC_VER)
// #error "MS compiler not supported"

#else
#warn "Unknown compiler"
#define LKT_PUSH
#define LKT_IGNORE_WARN_FMT_SECURITY
#define LKT_POP

#endif

/* Set the name of a thread, OS specific */
void lkt_thread_set_name(const char *name);

/* Get the path to the currently executing binary */
int lkt_read_self_exe(char *path, size_t len);

/* OS specific defines */
#define SELF_EXECUTABLE_LINUX   "/proc/self/exe"
#define SELF_EXECUTABLE_FREEBSD "/proc/curproc/file"
#define SELF_EXECUTABLE_SOLARIS "/proc/self/path/a.out"

#if defined(__linux__)
#if !defined(WCONTINUED)
#define WCONTINUED 0x00000008
#endif
#endif

/* Plateform specific functions */

/* Same as `be_uint64_t` but for Big-endian doubles. The result is a
   correct endian number. Restriction: bytes must be of length 8.
   Use the be_uint64_t defined function. */
double be_double_t(const uint8_t bytes[]);

/* Same as `be_double_t` but for floats. Restriction: bytes must
   be of length 4. The result is a correct endian number Use the
   be_uint32_t defined function. */
float be_float_t(const uint8_t bytes[]);

/* Read `bytes` as the big endian representation of a 32-bit unsigned integer.
   `n` is the number of bytes in `bytes`. The result is a 'correct'-endian.
   Returns 0 if n is 0.
   Restriction: n <= 4 */
uint32_t be_uint32_t(const uint8_t bytes[], size_t n);

/* Same as `be_uint32_t` but for 64-bit unsigned integers. The result is a
   'correct'-endian. Restriction: n <= 8 */
uint64_t be_uint64_t(const uint8_t bytes[], size_t n);

#if defined(__cplusplus)
}
#endif
