use crate::widgets;

/// Indicate which view should be displayed in the central panel of Amadeus'
/// main window.
#[derive(serde::Serialize, serde::Deserialize, PartialEq, Eq)]
pub enum AmadeusMainView {
    /// The main panel should show the queue of lektord, i.e. the kara that will
    /// be played.
    Queue,

    /// The main panel should show the kara that where played by lektord.
    Historic,

    /// The main panel should show the search view.
    SearchResults,
}

#[derive(serde::Serialize, serde::Deserialize)]
pub struct AmadeusConfig {
    pub dark_mode: bool,
    pub admin_password: String,
    pub lektord_hostname: String,
    pub lektord_port: super::NetworkPort,
    pub lektord_auto_reconnect: bool,
    pub main_panel_view: AmadeusMainView,
    pub side_panel_show: bool,
    pub ui_max_queue_items: super::IntegerTextBuffer<usize>,
    pub ui_max_search_items: super::IntegerTextBuffer<usize>,
    pub ui_max_history_items: super::IntegerTextBuffer<usize>,
}

impl Default for AmadeusMainView {
    fn default() -> Self {
        Self::Queue
    }
}

impl Default for AmadeusConfig {
    fn default() -> Self {
        Self {
            dark_mode: true,
            admin_password: "".to_owned(),
            lektord_hostname: "localhost".to_owned(),
            lektord_port: super::NetworkPort::from("6600"),
            lektord_auto_reconnect: false,
            main_panel_view: Default::default(),
            side_panel_show: true,
            ui_max_queue_items: super::IntegerTextBuffer::from("100"),
            ui_max_search_items: super::IntegerTextBuffer::from("100"),
            ui_max_history_items: super::IntegerTextBuffer::from("100"),
        }
    }
}

impl AmadeusConfig {
    pub fn render_settings_window(
        &mut self,
        ctx: &egui::Context,
        show: &mut bool,
        changed: &mut bool,
    ) {
        widgets::WindowBuilder::new("Settings", self.dark_mode).show(show, ctx, |ui| {
            ui.separator();
            widgets::add_heading2_label(ui, "☰  Lektord deamon");
            widgets::add_labelled_edit_line(
                ui,
                "Hostname         ",
                &mut self.lektord_hostname,
                changed,
            );
            widgets::add_labelled_edit_line(
                ui,
                "Port             ",
                &mut self.lektord_port,
                changed,
            );
            widgets::add_labelled_toggle_switch(
                ui,
                self.dark_mode,
                "Auto-reconnect",
                &mut self.lektord_auto_reconnect,
                changed,
            );

            ui.separator();
            widgets::add_heading2_label(ui, "☰  Admin user");
            widgets::add_labelled_password(
                ui,
                "Admin pwd        ",
                &mut self.admin_password,
                changed,
            );

            ui.separator();
            widgets::add_heading2_label(ui, "☰  UI");
            widgets::add_labelled_toggle_switch(
                ui,
                self.dark_mode,
                "Dark theme",
                &mut self.dark_mode,
                changed,
            );
            widgets::add_labelled_toggle_switch(
                ui,
                self.dark_mode,
                "Show side panel",
                &mut self.side_panel_show,
                changed,
            );
            widgets::add_labelled_edit_line(
                ui,
                "Max queue size   ",
                &mut self.ui_max_queue_items,
                changed,
            );
            widgets::add_labelled_edit_line(
                ui,
                "Max search size  ",
                &mut self.ui_max_search_items,
                changed,
            );
            widgets::add_labelled_edit_line(
                ui,
                "Max history size ",
                &mut self.ui_max_history_items,
                changed,
            );
        });
    }
}
